unit frmmain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  Menus, ActnList, ComCtrls, zipper;

type
  { TMainForm }

  TMainForm = class(TForm)
    AClose: TAction;
    AExtract: TAction;
    AOPen: TAction;
    AQuit: TAction;
    ALZip: TActionList;
    LVZip: TListView;
    MenuItem1: TMenuItem;
    MIExtract: TMenuItem;
    MIClose: TMenuItem;
    MISep1: TMenuItem;
    MIQuit: TMenuItem;
    MIOpen: TMenuItem;
    MMzip: TMainMenu;
    ODZip: TOpenDialog;
    SDExtract: TSelectDirectoryDialog;
    procedure AExtractExecute(Sender: TObject);
    procedure AExtractUpdate(Sender: TObject);
    procedure AOPenExecute(Sender: TObject);
    procedure AQuitExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FZip : TUnzipper;
    procedure ClearData;
    procedure ExtractToDir(ATargetDir: String);
    procedure OpenArchive(const AFileName: String);
    procedure ShowEntries;
    procedure ShowEntry(LI: TListItem; Z: TFullZipFileEntry);
    { private declarations }
  public
    { public declarations }
  end; 

var
  MainForm: TMainForm;

implementation

{ TMainForm }
resourcestring
  SViewingFile = 'Viewing zip file: %s';

procedure TMainForm.AQuitExecute(Sender: TObject);
begin
  ClearData;
  Close;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  LVZip.ViewStyle:=vsReport;
end;

procedure TMainForm.AOPenExecute(Sender: TObject);
begin
  If ODZip.Execute then
    OpenArchive(ODZip.FileName);
end;

procedure TMainForm.AExtractUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=(FZip<>Nil) and (LVZIP.SelCount>0);
end;

procedure TMainForm.AExtractExecute(Sender: TObject);
begin
  With SDExtract do
    if Execute then
      ExtractToDir(FileName);
end;

procedure TMainForm.ExtractToDir(ATargetDir : String);

Var
  L : TStrings;
  I : Integer;
  Z : TFullZipFileEntry;

begin
  FZip.OutputPath:=ATargetDir;
  L:=TStringList.Create;
  try
    For I:=0 to LVZip.Items.Count-1 do
      begin
      If LVZip.Items[i].Selected then
        begin
        Z:=TFullZipFileEntry(LVZip.Items[i].Data);
        L.Add(Z.ArchiveFileName);
        end;
      end;
    FZip.UnZipFiles(FZip.FileName,L);
  finally
    L.Free;
  end;
end;


procedure TMainForm.ClearData;

begin
  LVzip.Items.Clear;
  FreeAndNil(FZip);
end;

procedure TMainForm.OpenArchive(Const AFileName : String);

begin
  ClearData;
  FZip:=TUnZipper.Create;
  FZip.FileName:=AFileName;
  FZip.Examine;
  ShowEntries;
  Caption:=Format(SViewingFile,[AFileName]);
end;

procedure TMainForm.ShowEntries;

Var
  I : Integer;
  LI : TListItem;
begin
  LVZIP.Items.BeginUpdate;
  try
    For I:=0 to FZip.Entries.Count-1 do
      begin
      LI:=LVZip.Items.Add;
      ShowEntry(LI,FZip.Entries[i]);
      end;
  finally
    LVZIP.Items.EndUpdate;
  end;
end;

procedure TMainForm.ShowEntry(LI : TListItem; Z : TFullZipFileEntry);

Var
  S : String;

begin
  LI.Caption:=ExtractFileName(Z.ArchiveFileName);
  LI.Data:=Z;
  With LI.SubItems do
    begin
    BeginUpdate;
    try
      S:=ExtractFileExt(Z.ArchiveFileName);
      If (S='') then // Directory entry
        S:=Z.ArchiveFileName;
      Add(S);
      Add(IntToStr(Z.Size));
      Add(DateTimeToStr(Z.DateTime));
      Add(IntTostr(Z.CompressedSize));
      If (Z.Size=0) then
        Add('0')
      else
        Add(Format('%5.2f',[Z.CompressedSize/Z.Size*100]));
      Add(ExtractFilePath(Z.ArchiveFileName));
    finally
      EndUpdate;
    end;
    end;
end;



initialization
  {$I frmmain.lrs}

end.

