unit frmmaingrid;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  Menus, ActnList, ComCtrls, Grids, zipper;

type
  { TMainForm }

  TMainForm = class(TForm)
    AClose: TAction;
    AExtract: TAction;
    AOPen: TAction;
    AQuit: TAction;
    ALZip: TActionList;
    MenuItem1: TMenuItem;
    MIExtract: TMenuItem;
    MIClose: TMenuItem;
    MISep1: TMenuItem;
    MIQuit: TMenuItem;
    MIOpen: TMenuItem;
    MMzip: TMainMenu;
    ODZip: TOpenDialog;
    SDExtract: TSelectDirectoryDialog;
    SGZIp: TStringGrid;
    procedure AExtractExecute(Sender: TObject);
    procedure AExtractUpdate(Sender: TObject);
    procedure AOPenExecute(Sender: TObject);
    procedure AQuitExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FZip : TUnzipper;
    procedure ClearData;
    procedure ExtractToDir(ATargetDir: String);
    procedure OpenArchive(const AFileName: String);
    procedure ShowEntries;
    procedure ShowEntry(ARow : Integer; Z: TFullZipFileEntry);
    { private declarations }
  public
    { public declarations }
  end; 

var
  MainForm: TMainForm;

implementation

{ TMainForm }
resourcestring
  SViewingFile = 'Viewing zip file: %s';

procedure TMainForm.AQuitExecute(Sender: TObject);
begin
  ClearData;
  Close;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin

end;

procedure TMainForm.AOPenExecute(Sender: TObject);
begin
  If ODZip.Execute then
    OpenArchive(ODZip.FileName);
end;

procedure TMainForm.AExtractUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=(FZip<>Nil) and (SGZIP.Row>0);
end;

procedure TMainForm.AExtractExecute(Sender: TObject);
begin
  With SDExtract do
    if Execute then
      ExtractToDir(FileName);
end;

procedure TMainForm.ExtractToDir(ATargetDir : String);

Var
  L : TStrings;
  I : Integer;
  Z : TFullZipFileEntry;

begin
  FZip.OutputPath:=ATargetDir;
  L:=TStringList.Create;
  try
    For I:=SGZip.Selection.Top to SGZip.Selection.Bottom do
      begin
      Z:=TFullZipFileEntry(FZip.Entries[i-1]);
      L.Add(Z.ArchiveFileName);
      end;
    FZip.UnZipFiles(FZip.FileName,L);
  finally
    L.Free;
  end;
end;


procedure TMainForm.ClearData;

begin
  SGZIp.RowCount:=2;
  SGZip.Rows[1].CLear;
  FreeAndNil(FZip);
end;

procedure TMainForm.OpenArchive(Const AFileName : String);

begin
  ClearData;
  FZip:=TUnZipper.Create;
  FZip.FileName:=AFileName;
  FZip.Examine;
  ShowEntries;
  Caption:=Format(SViewingFile,[AFileName]);
end;

procedure TMainForm.ShowEntries;

Var
  I : Integer;
  LI : TListItem;
begin
  If FZip.Entries.Count=0 then
    Exit;
  SGZIP.RowCount:=FZip.Entries.Count+1;
  For I:=0 to FZip.Entries.Count-1 do
    ShowEntry(I+1,FZip.Entries[i]);
end;

procedure TMainForm.ShowEntry(ARow : Integer; Z : TFullZipFileEntry);

Var
  S : String;
  L : TStrings;

begin
  L:=TStringList.Create;
  try
    With L do
      begin
      S:=ExtractFileName(Z.ArchiveFileName);
      If (S='') then // Directory entry
        S:=Z.ArchiveFileName;
      Add(S);
      Add(ExtractFileExt(Z.ArchiveFileName));
      Add(IntToStr(Z.Size));
      Add(DateTimeToStr(Z.DateTime));
      Add(IntTostr(Z.CompressedSize));
      If (Z.Size=0) then
        Add('0')
      else
        Add(Format('%5.2f',[Z.CompressedSize/Z.Size*100]));
      Add(ExtractFilePath(Z.ArchiveFileName));
      end;
    SGZip.Rows[ARow]:=L;
  finally
    L.Free;
  end;
end;



initialization
  {$I frmmaingrid.lrs}

end.

